DECLARE SUB JDN2REC (DUMDATE AS ANY)
DECLARE FUNCTION ANGRDD# (X#)               'REDUCE ANGLES TO 0 TO 360
DECLARE FUNCTION ANGRDR# (X#)               'REDUCE ANGLES TO 0  TO 2*PI
DECLARE FUNCTION ATAN2# (S#, C#)            '2 ARGUMENT ARC TANGENT
DECLARE FUNCTION DT# (CENT#)                'DELTA T = ET - UT
DECLARE SUB EQUARG ()                       'UPDATE EQUILIB PHASE, NODE FACTS
DECLARE SUB HANDDH ()                       'COMPUTE HEIGHT/CURR & DERIVS
DECLARE SUB INITEL ()                       'INITIALIZE ELEMENT ARRAYS
DECLARE SUB PRASTRO ()                      'PRINT ASTRONOMICAL ARGUMENTS
DECLARE SUB PRCONST ()                      'PRINT CONSTITUENT FACTORS
DECLARE SUB PRTHDR (ILN%, IPH%)             'PRINT HEADER
DECLARE SUB UPDATE ()                       'UPDATE ASTRONOMICAL ARGUMENTS

'HGHTPR4A - PROCEDURES FOR COMPUTATION OF HEIGHT/CURRENT & DERIVATIVES

'FLAG FOR U.S. DAYLIGHT TIME, TFLAG, ADDED 1999 NOV 06


'ORBITAL ELEMENTS ------------------------------------------------------
'ELEM(1) = MEAN LONGITUDE OF MOON (s)
'ELEM(2) = MEAN LONGITUDE OF SUN  (h)
'ELEM(3) = MEAN LONGITUDE OF LUNAR PERIGEE (p)
'ELEM(4) = MEAN LONGITUDE OF SOLAR PERIGEE (p1)
'ELEM(5) = LONGITUDE OF MEAN ASCENDING NODE OF LUNAR ORBIT ON ECLIPTIC (N)
'ELEM(6) = OBLIQUITY OF THE ECLIPTIC (w)
'ELEM(7) = INCLINATION OF THE LUNAR ORBIT TO THE ECLIPTIC (i)
'ECCM    = ECCENTRICITY OF LUNAR ORBIT
'ECCS    = ECCENTRICITY OF SOLAR ORBIT
'SPRM    = SOLAR TO LUNAR TIDE RAISING FORCE

'DIMENSIONS, AND CONSTANTS ---------------------------------------------

  DEFINT I-N: DEFDBL A-H, O-Z
 
  TYPE DATEREC
    YEAR AS INTEGER
    MO AS INTEGER
    DA AS INTEGER
    LPGR AS INTEGER
    DATESTR AS STRING * 11
    DNOJ AS DOUBLE
    DOW AS STRING * 2
    CLNDR AS STRING * 3
    TFLAG AS INTEGER
  END TYPE

  CONST PI = 3.141592653589793#, RDPD = PI / 180#, DGPR = 180# / PI

  CONST NELEM = 7              'NUMBER OF ASTRONOMICAL ELEMENTS USED
  CONST ECCM = .054900489#     'ECCENTRICITY OF LUNAR ORBIT
  CONST ECCS = 1.675104E-02    'ECCENTRICITY OF SOLAR ORBIT
  CONST SPRM = .4602#          'SOLAR TO LUNAR TIDE RAISING FORCE
  CONST NCON = 70              'NUMBER OF   CONSTITUENTS ( NOT INCLUDING Z0 )
  CONST KINFR = 16             'NUMBER OF POSSIBLE INFERRED CONSTITUENTS

  COMMON SHARED TT, TDATE$, STRDATE0 AS DATEREC, STRDATE AS DATEREC, ENDDATE AS DATEREC, STTDST, ITRULE
  COMMON SHARED HEADER$(), DFNAM$

  COMMON SHARED FLDAZ, EBBAZ, TZ, Z0, KTIDTYP, AMPLI(), EPCHM(), INDXHC(), TITLE$, UNITS$
  COMMON SHARED NAM$(), SPEED(), PARAMS$, XUNITS$, EXTREMA$, COMMENTS$
  COMMON SHARED JCON, SMAMP, HT(), IINFR(), NOTES$

'FACTORS FOR ORBITAL ELEMENT EXPANSION AND CONSTITUENT FACTORS --------------
  DIM SHARED ELEMFC(NELEM, 4), ELEM(NELEM), ELEMDT(NELEM)
  DIM SHARED GKXI, GKNU, GKNP, GNPP, QU, RL2, CAPI, CK1, FK1AV, CK2, FK2AV
  DIM SHARED SINI, RAIL2, QAIM1

'CONSTITUENT ARRAYS ---------------------------------------------------------
  DIM SHARED NAM$(NCON), FCTND(NCON), SPEED(NCON), V0PLU(NCON)
  DIM SHARED AMPLI(NCON), EPCHM(NCON), INDXHC(NCON), ALFA(NCON), AMFC(NCON)

  DIM SHARED HT(4), IINFR(KINFR)
 
  'COEFFICENTS FOR ORBITAL ELEMENTS -- ELEMFC(I,J)
  DATA 270.434164, 13.1763965268, -8.500E-13, +3.90E-20
  DATA 279.696678,  0.9856473354, +2.267E-13, +0
  DATA 334.329556,  0.1114040803, -7.739E-12, -2.60E-19
  DATA 281.220833,  0.0000470684, +3.390E-13, +7.00E-20
  DATA 259.183275, -0.0529539222, +1.557E-12, -5.00E-20
  DATA  23.452294, -0.00000035626,-1.230E-15, +1.03E-20
  DATA   5.1453964, 0,             0,          0

  'NAME OF I-TH CONSTITUENT -- NAM$(I)
 
  DATA "M2"    ,"S2"    ,"N2"    ,"K1"    ,"M4"    ,"O1"    ,"M6"
  DATA "MK3"   ,"S4"    ,"MN4"   ,"NU2"   ,"S6"    ,"MU2"   ,"2N2"
  DATA "OO1"   ,"LDA2"  ,"S1"    ,"M1"    ,"J1"    ,"MM"    ,"SSA"
  DATA "SA"    ,"MSF"   ,"MF"    ,"RHO1"  ,"Q1"    ,"T2"    ,"R2"
  DATA "2Q1"   ,"P1"    ,"2SM2"  ,"M3"    ,"L2"    ,"2MK3"  ,"K2"
 
  DATA "M8"    ,"MS4"   ,"SIG1"  ,"MP1"   ,"NO1"   ,"CHI1"  ,"PI1"
  DATA "PSI1"  ,"PHI1"  ,"THE1"  ,"SO1"   ,"OQ2"   ,"MNS2"  ,"OP2"
  DATA "MKS2"  ,"MSN2"  ,"KJ2"   ,"MO3"   ,"SO3"   ,"SK3"   ,"SN4"
  DATA "MK4"   ,"SK4"   ,"2MN6"  ,"MSN6"  ,"2MS6"  ,"2MK6"  ,"2SM6"
  DATA "MSK6"  ,"MSM"   ,"MTM"   ,"KQ1"   ,"S3"    ,"O2"    ,"2MN2"

  'INDEX OF K-TH INFERABLE CONSTITUENT -- IINFR(K)
  DATA 03, 11, 13, 14, 15,   16, 18, 19, 25, 26,   27, 28, 29, 30, 33,   35

'END HGHTPRC4.MOD =========================================================

FUNCTION DT (CENT)
  'COMPUTE DELTA T = ET - UT -------------------------------------------------
  'DELTA T FROM STEPHENSON AND MORRISON, ROTATION OF THE EARTH: 700 B.C TO
  ' A.D. 1980, PHIL. TRANS. R. SOC LOND. A 313, 47-70, (1984)
  'SEE FIG. 5 FOR MODERN, PAGE 68 FOR ANCIENT DT - SWITCH @ EARLY INTERSECTION
  IF (CENT > -11.7495) THEN
    DTS = -17.2 + 25.5 * (CENT + .8) ^ 2        'DELTA T IN SECONDS
  ELSE
    DTS = 1850 + (-435 + 44.3 * (CENT + 9.52)) * (CENT + 9.52)
  END IF
  DT = DTS / 86400                           'DELTA T IN DAYS
END FUNCTION 'DT ============================================================

SUB EQUARG
  'COMPUTATION OF EQUILIBRIUM ARGUMENTS AND SPEEDS - SEE MANUAL, TABLE 2
  'SEE ALSO PAGE 48 RE MSf - EQUILIB. ARG. AND NODE FACTOR CHANGED 1992FEB08
  'TO VALUES FOR S2-M2 COMPOUND TIDE
  '
  'FIRST 37 CONSTITUENTS USED BY NOS TIDE PREDICTING MACHINE NO. 2
  'NEXT  27 CONSTITUENTS USED BY IHO DATA BANK - 1988 DATA SUBMISSION FORM
  'NEXT   4 CONSTITUENTS USED BY ANALES DEL INSTITUTO DE GEOFISICA
  'NEXT   2 CONSTITUENTS USED BY DEUTSCHE HYDROGRAPHISCHE INSTITUT
  '

  TDT = 15 * RDPD: T = ANGRDR(12 * TDT)     'HR. ANG. MEAN SUN = 180 @ O HRS
  S = ELEM(1): SDT = ELEMDT(1)
  H = ELEM(2): HDT = ELEMDT(2)
  P = ELEM(3): PDT = ELEMDT(3)
  P1 = ELEM(4): P1DT = ELEMDT(4)
  SMLI = ELEM(7): OMEGA = ELEM(6): EN = ELEM(5)

  V0PLU(1) = 2 * T - 2 * S + 2 * H + 2 * GKXI - 2 * GKNU           'M2
  SPEED(1) = 2 * TDT - 2 * SDT + 2 * HDT
  V0PLU(2) = 2 * T                                                 'S2
  SPEED(2) = 2 * TDT
  V0PLU(3) = 2 * T - 3 * S + 2 * H + P + 2 * GKXI - 2 * GKNU       'N2
  SPEED(3) = 2 * TDT - 3 * SDT + 2 * HDT + PDT
  V0PLU(4) = T + H - PI / 2 - GKNP                                 'K1
  SPEED(4) = TDT + HDT
  V0PLU(5) = 4 * T - 4 * S + 4 * H + 4 * GKXI - 4 * GKNU           'M4
  SPEED(5) = 4 * TDT - 4 * SDT + 4 * HDT
  V0PLU(6) = T - 2 * S + H + PI / 2 + 2 * GKXI - GKNU              'O1
  SPEED(6) = TDT - 2 * SDT + HDT
  V0PLU(7) = 6 * T - 6 * S + 6 * H + 6 * GKXI - 6 * GKNU           'M6
  SPEED(7) = 6 * TDT - 6 * SDT + 6 * HDT
  V0PLU(8) = 3 * T - 2 * S + 3 * H - PI / 2 + 2 * GKXI - 2 * GKNU - GKNP  'MK3
  SPEED(8) = 3 * TDT - 2 * SDT + 3 * HDT
  V0PLU(9) = 4 * T                                                 'S4
  SPEED(9) = 4 * TDT
  V0PLU(10) = 4 * T - 5 * S + 4 * H + P + 4 * GKXI - 4 * GKNU      'MN4
  SPEED(10) = 4 * TDT - 5 * SDT + 4 * HDT + PDT
  V0PLU(11) = 2 * T - 3 * S + 4 * H - P + 2 * GKXI - 2 * GKNU      'NU2
  SPEED(11) = 2 * TDT - 3 * SDT + 4 * HDT - PDT
  V0PLU(12) = 6 * T                                                'S6
  SPEED(12) = 6 * TDT
  V0PLU(13) = 2 * T - 4 * S + 4 * H + 2 * GKXI - 2 * GKNU          'MU2
  SPEED(13) = 2 * TDT - 4 * SDT + 4 * HDT
  V0PLU(14) = 2 * T - 4 * S + 2 * H + 2 * P + 2 * GKXI - 2 * GKNU  '2N2
  SPEED(14) = 2 * TDT - 4 * SDT + 2 * HDT + 2 * PDT
  V0PLU(15) = T + 2 * S + H - PI / 2 - 2 * GKXI - GKNU             'OO1
  SPEED(15) = TDT + 2 * SDT + HDT
  V0PLU(16) = 2 * T - S + P + PI + 2 * GKXI - 2 * GKNU             'LDA2
  SPEED(16) = 2 * TDT - SDT + PDT
  V0PLU(17) = T                                                    'S1
  SPEED(17) = TDT
  V0PLU(18) = T - S + H + P - PI / 2 - GKNU - QU                   'M1
  SPEED(18) = TDT - SDT + HDT + PDT
  V0PLU(19) = T + S + H - P - PI / 2 - GKNU                        'J1
  SPEED(19) = TDT + SDT + HDT - PDT
  V0PLU(20) = S - P                                                'MM
  SPEED(20) = SDT - PDT
  V0PLU(21) = 2 * H                                                'SSA
  SPEED(21) = 2 * HDT
  V0PLU(22) = H                                                    'SA
  SPEED(22) = HDT
  V0PLU(23) = 2 * S - 2 * H - 2 * GKXI + 2 * GKNU                  'MSF P.48
  SPEED(23) = 2 * SDT - 2 * HDT
  V0PLU(24) = 2 * S - 2 * GKXI                                     'MF
  SPEED(24) = 2 * SDT
  V0PLU(25) = T - 3 * S + 3 * H - P + PI / 2 + 2 * GKXI - GKNU     'RHO1
  SPEED(25) = TDT - 3 * SDT + 3 * HDT - PDT
  V0PLU(26) = T - 3 * S + H + P + PI / 2 + 2 * GKXI - GKNU         'Q1
  SPEED(26) = TDT - 3 * SDT + HDT + PDT
  V0PLU(27) = 2 * T - H + P1                                       'T2
  SPEED(27) = 2 * TDT - HDT + P1DT
  V0PLU(28) = 2 * T + H - P1 + PI                                  'R2
  SPEED(28) = 2 * TDT + HDT - P1DT
  V0PLU(29) = T - 4 * S + H + 2 * P + PI / 2 + 2 * GKXI - GKNU     '2Q1
  SPEED(29) = TDT - 4 * SDT + HDT + 2 * PDT
  V0PLU(30) = T - H + PI / 2                                       'P1
  SPEED(30) = TDT - HDT
  V0PLU(31) = 2 * T + 2 * S - 2 * H - 2 * GKXI + 2 * GKNU          '2SM2
  SPEED(31) = 2 * TDT + 2 * SDT - 2 * HDT
  V0PLU(32) = 3 * T - 3 * S + 3 * H + 3 * GKXI - 3 * GKNU          'M3
  SPEED(32) = 3 * TDT - 3 * SDT + 3 * HDT
  V0PLU(33) = 2 * T - S + 2 * H - P + PI + 2 * GKXI - 2 * GKNU - RL2'L2
  SPEED(33) = 2 * TDT - SDT + 2 * HDT - PDT
  V0PLU(34) = 3 * T - 4 * S + 3 * H + PI / 2 + 4 * GKXI - 4 * GKNU + GKNP '2MK3
  SPEED(34) = 3 * TDT - 4 * SDT + 3 * HDT
  V0PLU(35) = 2 * T + 2 * H - 2 * GNPP                             'K2
  SPEED(35) = 2 * TDT + 2 * HDT
  V0PLU(36) = 8 * T - 8 * S + 8 * H + 8 * GKXI - 8 * GKNU          'M8
  SPEED(36) = 8 * TDT - 8 * SDT + 8 * HDT
  V0PLU(37) = 4 * T - 2 * S + 2 * H + 2 * GKXI - 2 * GKNU          'MS4
  SPEED(37) = 4 * TDT - 2 * SDT + 2 * HDT
  V0PLU(38) = T - 4 * S + 3 * H + PI / 2 + 2 * GKXI - GKNU         'SIG1
  SPEED(38) = TDT - 4 * SDT + 3 * HDT
  V0PLU(39) = T - 2 * S + 3 * H - PI / 2 - GKNU                    'MP1
  SPEED(39) = TDT - 2 * SDT + 3 * HDT
  V0PLU(40) = T - S + H + P - PI / 2 - GKNU                        'NO1
  SPEED(40) = TDT - SDT + HDT + PDT
  V0PLU(41) = T - S + 3 * H - P - PI / 2 - GKNU                    'CHI1
  SPEED(41) = TDT - SDT + 3 * HDT - PDT
  V0PLU(42) = T - 2 * H + P1 + PI / 2                              'PI1
  SPEED(42) = TDT - 2 * HDT + P1DT
  V0PLU(43) = T + 2 * H - P1 - PI / 2                              'PSI1
  SPEED(43) = TDT + 2 * HDT - P1DT
  V0PLU(44) = T + 3 * H - PI / 2                                   'PHI1
  SPEED(44) = TDT + 3 * HDT
  V0PLU(45) = T + S - H + P - PI / 2 - GKNU                        'THE1
  SPEED(45) = TDT + SDT - HDT + PDT
  V0PLU(46) = T + 2 * S - H - PI / 2 - GKNU                        'SO1
  SPEED(46) = TDT + 2 * SDT - HDT
  V0PLU(47) = 2 * T - 5 * S + 2 * H + 3 * P     'ADD U             'OQ2
  SPEED(47) = 2 * TDT - 5 * SDT + 2 * HDT + 3 * PDT
  V0PLU(48) = 2 * T - 5 * S + 4 * H + P + 4 * GKXI - 4 * GKNU      'MNS2
  SPEED(48) = 2 * TDT - 5 * SDT + 4 * HDT + PDT
  V0PLU(49) = 2 * T - 2 * S + PI + 2 * GKXI - GKNU                 'OP2
  SPEED(49) = 2 * TDT - 2 * SDT
  V0PLU(50) = 2 * T - 2 * S + 4 * H + 2 * GKXI - 2 * GKNU - 2 * GNPP 'MKS2
  SPEED(50) = 2 * TDT - 2 * SDT + 4 * HDT
  V0PLU(51) = 2 * T + S - P                                        'MSN2
  SPEED(51) = 2 * TDT + SDT - PDT
  V0PLU(52) = 2 * T + S + 2 * H - P - 2 * GKNU                     'KJ2
  SPEED(52) = 2 * TDT + SDT + 2 * HDT - PDT
  V0PLU(53) = 3 * T - 4 * S + 3 * H + PI / 2 + 4 * GKXI - 3 * GKNU 'MO3
  SPEED(53) = 3 * TDT - 4 * SDT + 3 * HDT
  V0PLU(54) = 3 * T - 2 * S + H + PI / 2 + 2 * GKXI - GKNU         'SO3
  SPEED(54) = 3 * TDT - 2 * SDT + HDT
  V0PLU(55) = 3 * T + H - PI / 2 - GKNP                            'SK3
  SPEED(55) = 3 * TDT + HDT
  V0PLU(56) = 4 * T - 3 * S + 2 * H + P + 2 * GKXI - 2 * GKNU      'SN4
  SPEED(56) = 4 * TDT - 3 * SDT + 2 * HDT
  V0PLU(57) = 4 * T - 2 * S + 4 * H + 2 * GKXI - 2 * GKNU - 2 * GNPP 'MK4
  SPEED(57) = 4 * TDT - 2 * SDT + 4 * HDT
  V0PLU(58) = 4 * T + 2 * H - 2 * GNPP                             'SK4
  SPEED(58) = 4 * TDT + 2 * HDT
  V0PLU(59) = 6 * T - 7 * S + 6 * H + P + 6 * GKXI - 6 * GKNU      '2MN6
  SPEED(59) = 6 * TDT - 7 * SDT + 6 * HDT + PDT
  V0PLU(60) = 6 * T - 5 * S + 4 * H + P + 4 * GKXI - 4 * GKNU      'MSN6
  SPEED(60) = 6 * TDT - 5 * SDT + 4 * HDT + PDT
  V0PLU(61) = 6 * T - 4 * S + 4 * H + 4 * GKXI - 4 * GKNU          '2MS6
  SPEED(61) = 6 * TDT - 4 * SDT + 4 * HDT
  V0PLU(62) = 6 * T - 4 * S + 6 * H + 4 * GKXI - 4 * GKNU - 2 * GNPP '2MK6
  SPEED(62) = 6 * TDT - 4 * SDT + 6 * HDT
  V0PLU(63) = 6 * T - 2 * S + 2 * H + 2 * GKXI - 2 * GKNU          '2SM6
  SPEED(63) = 6 * TDT - 2 * SDT + 2 * HDT
  V0PLU(64) = 6 * T - 2 * S + 4 * H + 2 * GKXI - 2 * GKNU - 2 * GNPP 'MSK6
  SPEED(64) = 6 * TDT - 2 * SDT + 4 * HDT
  V0PLU(65) = S - 2 * H + P                                        'MSM = A4
  SPEED(65) = SDT - 2 * HDT + PDT
  V0PLU(66) = 3 * S - P - 2 * GKXI                                 'MTM = A7
  SPEED(66) = 3 * SDT - PDT
  V0PLU(67) = T + 3 * S + H - P - PI / 2 - 2 * GKXI - GKNU         'KQ1
  SPEED(67) = TDT + 3 * SDT + HDT - PDT
  V0PLU(68) = 3 * T                                                'S3
  SPEED(68) = 3 * TDT
  V0PLU(69) = 2 * T - 4 * S + 2 * H + PI + 4 * GKXI - 2 * GKNU     'O1
  SPEED(69) = 2 * TDT - 4 * SDT + 2 * HDT
  V0PLU(70) = 2 * T - S + 2 * H - P + 2 * GKXI - GKNU              '2MN2
  SPEED(70) = 2 * TDT - SDT + 2 * HDT - PDT
 
  FOR I = 1 TO NCON
    V0PLU(I) = ANGRDR(V0PLU(I))
  NEXT I

  'COMPUTATION OF NODE FACTORS ----------------------------------------
  FCTND(1) = (COS(CAPI / 2) / (COS(OMEGA / 2) * COS(SMLI / 2))) ^ 4
  FCTND(2) = 1
  FCTND(3) = FCTND(1)
  FCTND(4) = CK1 / FK1AV
  FCTND(5) = FCTND(1) ^ 2
  FCTND(6) = (SINI / SIN(OMEGA)) * (COS(CAPI / 2) / (COS(OMEGA / 2) * COS(SMLI / 2) ^ 2)) ^ 2
  FCTND(7) = FCTND(1) ^ 3
  FCTND(8) = FCTND(1) * FCTND(4)
  FCTND(9) = 1
  FCTND(10) = FCTND(5)
  FCTND(11) = FCTND(1)
  FCTND(12) = 1
  FCTND(13) = FCTND(1)
  FCTND(14) = FCTND(1)
  FCTND(15) = (SINI / SIN(OMEGA)) * (SIN(CAPI / 2) / (SIN(OMEGA / 2) * COS(SMLI / 2) ^ 2)) ^ 2
  FCTND(16) = FCTND(1)
  FCTND(17) = 1
  FCTND(18) = FCTND(6) * QAIM1
  FCTND(19) = SIN(2 * CAPI) / (SIN(2 * OMEGA) * (1 - 1.5# * SIN(SMLI) ^ 2))
  FCTND(20) = (2 / 3# - SINI ^ 2) / ((2 / 3# - SIN(OMEGA) ^ 2) * (1 - 1.5# * SIN(SMLI) ^ 2))
  FCTND(21) = 1
  FCTND(22) = 1
  FCTND(23) = FCTND(1)                                           'SEE P. 48
  FCTND(24) = SINI ^ 2 / (SIN(OMEGA) * COS(SMLI / 2) ^ 2) ^ 2
  FCTND(25) = FCTND(6)
  FCTND(26) = FCTND(6)
  FCTND(27) = 1
  FCTND(28) = 1
  FCTND(29) = FCTND(6)
  FCTND(30) = 1
  FCTND(31) = FCTND(1)
  FCTND(32) = FCTND(1) ^ 1.5
  FCTND(33) = FCTND(1) * RAIL2
  FCTND(34) = FCTND(1) ^ 2 * FCTND(4)
  FCTND(35) = CK2 / FK2AV
  FCTND(36) = FCTND(1) ^ 4
  FCTND(37) = FCTND(1)
  FCTND(38) = FCTND(6)
  FCTND(39) = FCTND(19)
  FCTND(40) = FCTND(3) * FCTND(6)
  FCTND(41) = FCTND(19)
  FCTND(42) = 1
  FCTND(43) = 1
  FCTND(44) = 1
  FCTND(45) = FCTND(19)
  FCTND(46) = FCTND(19)
  FCTND(47) = 1    'MODIFY PER CANADIAN
  FCTND(48) = FCTND(1) * FCTND(3)
  FCTND(49) = FCTND(6) * FCTND(30)
  FCTND(50) = FCTND(1) * FCTND(35)
  FCTND(51) = FCTND(1) * FCTND(3)
  FCTND(52) = FCTND(35)
  FCTND(53) = FCTND(1) * FCTND(6)
  FCTND(54) = FCTND(6)
  FCTND(55) = FCTND(4)
  FCTND(56) = FCTND(3)
  FCTND(57) = FCTND(1) * FCTND(35)
  FCTND(58) = FCTND(35)
  FCTND(59) = FCTND(1) ^ 2 * FCTND(3)
  FCTND(60) = FCTND(1) * FCTND(3)
  FCTND(61) = FCTND(1) ^ 2
  FCTND(62) = FCTND(1) ^ 2 * FCTND(35)
  FCTND(63) = FCTND(1)
  FCTND(64) = FCTND(1) * FCTND(35)
  FCTND(65) = FCTND(20)
  FCTND(66) = FCTND(24)
  FCTND(67) = FCTND(15)
  FCTND(68) = 1
  FCTND(69) = FCTND(6) ^ 2
  FCTND(70) = FCTND(1) ^ 2 * FCTND(3)

  'COMPUTE CONSTITUENT AMPLITUDES, SUMS AND PHASES --------------------------
  SMAMP = 0                             'SUM OF AMPLITUDES
  FOR I = 1 TO NCON
    AMFC(I) = AMPLI(I) * FCTND(I)
    SMAMP = SMAMP + AMFC(I)
    ALFA(I) = V0PLU(I) - RDPD * EPCHM(I)
  NEXT I
  IF (KTIDTYP = 2) THEN SMAMP = SQR(SMAMP)
END SUB 'EQUARG =============================================================

SUB HANDDH
  'COMPUTE HEIGHT OF TIDE OR CURRENT -- SEE MANUAL, PAGES 126 & 150 ------
  'TT = TIME FROM START OF INTERVAL, IN HOURS, STANDARD OR DAYLIGHT
  'TT - STTDST = LOCAL STANDARD TIME USED IN HEIGHT COMPUTATION
  
  IF (ABS(TT) > 720) THEN                   'MORE THAN 30 DAYS FROM START
    DAYS = INT(TT / 24)
    TT = TT - 24 * DAYS
    STRDATE.DNOJ = STRDATE.DNOJ + DAYS
    CALL JDN2REC(STRDATE)
    CALL UPDATE
    CALL EQUARG
  END IF
               
  HT(1) = 0: HT(2) = 0: HT(3) = 0
  FOR JH = 1 TO JCON
    IH = INDXHC(JH)
    AMP = AMFC(IH)                             'AMFC = NODEFACTOR * AMPLITUDE
    IF (AMP > 0) THEN
      ARG = ANGRDR((TT - STTDST) * SPEED(IH) + ALFA(IH))  'ALFA = EQUILIB. ARG + EPCHM
      HTI = AMP * COS(ARG)
      HT(1) = HT(1) + HTI
      HT(2) = HT(2) - AMP * SIN(ARG) * SPEED(IH)
      HT(3) = HT(3) - HTI * SPEED(IH) * SPEED(IH)
    END IF
  NEXT JH
  IF (KTIDTYP = 2) THEN HT(1) = SGN(HT(1)) * SQR(ABS(HT(1)))
  HT(1) = HT(1) + Z0

END SUB '===================================================================

SUB INITEL
 
  'READ IN COEFFICIENTS FOR ASTRONOMICAL ELEMENTS
  FOR I = 1 TO NELEM: FOR J = 1 TO 4: READ ELEMFC(I, J): NEXT J: NEXT I

  'READ IN CONSTITUENT NAMES
  FOR I = 1 TO NCON: READ NAM$(I): NEXT I

  'READ IN INDICES OF INFERABLE CONSTITUENTS
  FOR K = 1 TO KINFR: READ IINFR(K): NEXT K

END SUB 'INITEL ===========================================================

SUB PRASTRO
  'PRINT ASTRONOMICAL FACTORS ---------------------------------------------
  CLS
  CALL PRTHDR(1, 0)
  PRINT "     ELEMENTS EVALUATED AT START  OF INTERVAL - ";
  PRINT USING "JD =#########.##"; STRDATE.DNOJ;
  PRINT USING ",\         \"; STRDATE.DATESTR: PRINT
  PRINT "     MEAN LONGITUDE OF MOON           (s)   ";
  PRINT USING "     ####.######"; ELEM(1) / RDPD
  PRINT "     MEAN LONGITUDE OF SUN            (h)   ";
  PRINT USING "     ####.######"; ELEM(2) / RDPD
  PRINT "     MEAN LONGITUDE OF LUNAR PERIGEE  (p)   ";
  PRINT USING "     ####.######"; ELEM(3) / RDPD
  PRINT "     MEAN LONGITUDE OF SOLAR PERIGEE  (p1)  ";
  PRINT USING "     ####.######"; ELEM(4) / RDPD
  PRINT
  PRINT "     ELEMENTS EVALUATED AT MIDDLE OF INTERVAL - ";
  PRINT USING "JD =#########.##"; STRDATE.DNOJ + 15: PRINT
  PRINT "     LONGITUDE OF NODE OF LUNAR ORBIT (N)   ";
  PRINT USING "     ####.######"; ELEM(5) / RDPD
  PRINT "     OBLIQUITY OF ECLIPTIC            (w)   ";
  PRINT USING "     ####.######"; ELEM(6) / RDPD
  PRINT "     INCLINATION OF LUNAR ORBIT       (i)   ";
  PRINT USING "     ####.######"; ELEM(7) / RDPD
  PRINT
  PRINT "     OBLIQUITY OF LUNAR ORBIT         (I)   ";
  PRINT USING "     ####.######"; CAPI / RDPD
  PRINT "     RT. ASC. OF INTERSECTION         (NU)  ";
  PRINT USING "     ####.######"; GKNU / RDPD
  PRINT "     LONG. IN ORBIT OF INTERSECTION   (XI)  ";
  PRINT USING "     ####.######"; GKXI / RDPD
  PRINT "     CONSTITUENT M1 ARGUMENT TERM     (QU)  ";
  PRINT USING "     ####.######"; QU / RDPD
  PRINT "     CONSTITUENT L2 ARGUMENT TERM     (R)   ";
  PRINT USING "     ####.######"; RL2 / RDPD
  PRINT "     CONSTITUENT L2 NODE     TERM     (1/RA)";
  PRINT USING "     ####.######"; RAIL2
  PRINT "     LUNISOLAR   K1 ARGUMENT TERM     (NU') ";
  PRINT USING "     ####.######"; GKNP / RDPD
  PRINT "     CONSTITUENT K2 ARGUMENT TERM     (NU'')";
  PRINT USING "     ####.######"; GNPP / RDPD
  LOCATE 25, 10: INPUT ; "     PRINT UPDATED ELEMENTS (Y/N) ?  ", ANS$
  ANS$ = UCASE$(LEFT$(ANS$, 1))
  IF (ANS$ = "Y") THEN
    'PRINT PAGE - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    FOR ILINE = 1 TO 24
      FOR JCOL = 1 TO 80
        PRINT #3, CHR$(SCREEN(ILINE, JCOL, 0));
      NEXT JCOL
      PRINT #3, ""
    NEXT ILINE
    PRINT #3, CHR$(12)
  END IF
END SUB ' PRASTRO =========================================================

SUB PRCONST
  'PRINT FACTORS FOR CONSTITUENTS -------------------------------------------
  CLS
  CALL PRTHDR(2, 0)
  LOCATE 10, 20: INPUT "PRINT CONSTITUENT FACTORS (Y/N) ? ", ANS$
  ANS$ = UCASE$(LEFT$(ANS$, 1))
  IF (ANS$ = "Y") THEN IP2 = 1 ELSE IP2 = 0

  CLS
  CALL PRTHDR(2, IP2)
  IF (NOTES$ <> "") THEN PRINT NOTES$: PRINT
  PRINT COMMENTS$: PRINT
  PRINT "      CONSTITUENT FACTORS FOR "; STRDATE.DATESTR; " - "; ENDDATE.DATESTR
  IF ((KTIDTYP > 0) AND (KTIDTYP < 3)) THEN
    PRINT : PRINT USING "      DIRECTION AT FLOOD(+) = ###, AT EBB(-) = ###"; FLDAZ; EBBAZ;
    IF KTIDTYP = 2 THEN PRINT "  - HYDRAULIC CURRENT" ELSE PRINT
  END IF
  PRINT : PRINT "      Z0 = "; : PRINT USING "###.####"; Z0;
  PRINT "     KTIDTYP = "; : PRINT USING "#"; KTIDTYP;
  PRINT "     "; UNITS$; " UNITS";
  PRINT "   TIME ZONE = "; : PRINT USING "+##.## "; TZ: PRINT
  PRINT "      CONST.   B/H      K'        EQULB ARG    NODE FCT   SPEED"
  PRINT
  IF (IP2 > 0) THEN
    IF (NOTES$ <> "") THEN PRINT #3, NOTES$: PRINT #3, ""
    PRINT #3, COMMENTS$: PRINT #3, ""
    PRINT #3, "      CONSTITUENT FACTORS FOR "; STRDATE.DATESTR; " - "; ENDDATE.DATESTR
    IF ((KTIDTYP > 0) AND (KTIDTYP < 3)) THEN
      PRINT #3, "": PRINT #3, USING "      DIRECTION AT FLOOD(+) = ###, AT EBB(-) = ###"; FLDAZ; EBBAZ;
      IF KTIDTYP = 2 THEN PRINT #3, "  - HYDRAULIC CURRENT" ELSE PRINT #3, ""
    END IF
    PRINT #3, "": PRINT #3, "      Z0 = "; : PRINT #3, USING "###.####"; Z0;
    PRINT #3, "     KTIDTYP = "; : PRINT #3, USING "#"; KTIDTYP;
    PRINT #3, "     "; UNITS$; " UNITS";
    PRINT #3, "   TIME ZONE = "; : PRINT #3, USING "+##.## "; TZ: PRINT #3, ""
    PRINT #3, "      CONST.    B/H      K'        EQULB ARG    NODE FCT   SPEED"
    PRINT #3, ""
  END IF

  LINENO = 11
  FOR J = 1 TO JCON
    I = INDXHC(J)
    PRINT USING "      \     \  "; NAM$(I);
    PRINT USING "##.####  "; AMPLI(I);
    PRINT USING "###.##    "; EPCHM(I);
    PRINT USING "###.####    "; V0PLU(I) / RDPD;
    PRINT USING "##.#####    "; FCTND(I);
    PRINT USING "###.#######"; SPEED(I) / RDPD;
    IF (IP2 > 0) THEN
      PRINT #3, USING "      \      \  "; NAM$(I);
      PRINT #3, USING "##.####  "; AMPLI(I);
      PRINT #3, USING "###.##    "; EPCHM(I);
      PRINT #3, USING "###.####    "; V0PLU(I) / RDPD;
      PRINT #3, USING "##.#####    "; FCTND(I);
      PRINT #3, USING "###.#######"; SPEED(I) / RDPD
    END IF
    IF (CSRLIN = 24) THEN
      LOCATE 25, 27: PRINT "PRESS ANY KEY TO CONTINUE.";
      WHILE (INKEY$ = ""): WEND
      IF (J < JCON) THEN
        CLS
        PRINT "      CONST.   B/H      K'        EQULB ARG    NODE FCT   SPEED"
        PRINT
        LINENO = 3
      ELSE
        EXIT SUB
      END IF
    ELSE
      PRINT
    END IF
  NEXT J
  IF (IP2 > 0) THEN PRINT #3, CHR$(12)
  LOCATE 25, 27: PRINT "PRESS ANY KEY TO CONTINUE.";
  WHILE (INKEY$ = ""): WEND
END SUB 'PRCONST ==========================================================

SUB UPDATE
  'UPDATE ORBITAL ELEMENTS AND COMPUTE AUXILIARY FACTORS ------------------

  CONST DNOJ1 = 2415020#        '1900 JAN 0.5 - ORIGIN OF T FOR EPHEMERIDES

  FOR I = 1 TO 7: ELEM(I) = 0: ELEMDT(I) = 0: NEXT I
 
  DD = STRDATE.DNOJ - DNOJ1     'UPDATE ELEMENTS 1-4 TO START OF INTERVAL
  CENT = DD / 36525             'CENTURIES SINCE DNOJ1
  DD = DD + DT(CENT)            'CONVERT DD TO EPHEMERIS TIME
  FOR I = 1 TO 4
    FOR J = 4 TO 2 STEP -1
      ELEM(I) = ELEMFC(I, J) + DD * ELEM(I)
      ELEMDT(I) = (J - 1) * ELEMFC(I, J) + DD * ELEMDT(I)
    NEXT J
    ELEM(I) = ELEMFC(I, 1) + DD * ELEM(I)
    ELEM(I) = ANGRDD(ELEM(I))
  NEXT I
 
  DD = DD + 15                  'UPDATE ELEMENTS 5-7 TO MID-INTERVAL
  FOR I = 5 TO 7
    FOR J = 4 TO 2 STEP -1
      ELEM(I) = ELEMFC(I, J) + DD * ELEM(I)
      ELEMDT(I) = (J - 1) * ELEMFC(I, J) + DD * ELEMDT(I)
    NEXT J
    ELEM(I) = ELEMFC(I, 1) + DD * ELEM(I)
    ELEM(I) = ANGRDD(ELEM(I))
  NEXT I
 
  'CONVERT ANGLES TO RADIANS AND RATES TO RADIANS PER HOUR ------------
  FOR I = 1 TO NELEM
    ELEM(I) = RDPD * ELEM(I)
    ELEMDT(I) = RDPD * ELEMDT(I) / 24
  NEXT I

  'COMPUTE OBLIQUITY WRT EQUATOR AND RA OF LUNAR INTERSECTION, ETC. --
  'SEE FIGURE 1, PAGE 6 OF MANUAL
  SMLI = ELEM(7): OMEGA = ELEM(6): EN = ELEM(5)
 
  'OBLIQUITY OF LUNAR ORBIT WITH RESPECT TO EARTH'S EQUATOR
  COSI = COS(OMEGA) * COS(SMLI) - SIN(OMEGA) * SIN(SMLI) * COS(EN)
  SINI = SQR(1 - COSI ^ 2)
  CAPI = ATAN2(SINI, COSI)
 
  'RIGHT ASCENSION OF LUNAR INTERSECTION
  SINN = SIN(SMLI) * SIN(EN) / SINI
  COSN = SQR(1 - SINN ^ 2)
  GKNU = ATAN2(SINN, COSN)
 
  'LONGITUDE IN MOON'S ORBIT OF LUNAR INTERSECTION
  SINA = SIN(OMEGA) * SIN(EN) / SINI
  COSA = COS(EN) * COSN + SIN(EN) * SINN * COS(OMEGA)
  ALF1 = ATAN2(SINA, COSA)
  GKXI = EN - ALF1
 
  'COMPUTE K1 AND K2 FACTORS - MANUAL, PAGE 45 --------------------------
  XK1M = .5# + .75# * ECCM ^ 2
  XK1S = SPRM * (.5# + .75# * ECCS ^ 2)
  AK1 = XK1M * SIN(2 * CAPI)
  BK1 = XK1S * SIN(2 * OMEGA)
  GKNP = ATAN2(AK1 * SINN, AK1 * COSN + BK1)
  CK1 = SQR(AK1 ^ 2 + 2 * AK1 * BK1 * COSN + BK1 ^ 2)
  FK1AV = SIN(2 * OMEGA) * (XK1M * (1 - 1.5 * SIN(SMLI) ^ 2) + XK1S)
  AK2 = XK1M * SINI ^ 2
  BK2 = XK1S * (SIN(OMEGA)) ^ 2
  GNPP = .5 * ATAN2(AK2 * SIN(2 * GKNU), AK2 * COS(2 * GKNU) + BK2)
  CK2 = SQR(AK2 ^ 2 + 2 * AK2 * BK2 * COS(2 * GKNU) + BK2 ^ 2)
  FK2AV = SIN(OMEGA) ^ 2 * (XK1M * (1 - 1.5 * SIN(SMLI) ^ 2) + XK1S)
 
  'COMPUTE L2 FACTORS - SEE P. 43 REF. 1 --------------------------------
  'FIRST COMPUTE PERIGEE-INTERSECTION, P, AT START OF INTERVAL
  PL2 = ELEM(3) - GKXI
  SR = 6 * SIN(2 * PL2) * (SIN(CAPI / 2)) ^ 2
  CR = (COS(CAPI / 2)) ^ 2 - 6 * COS(2 * PL2) * (SIN(CAPI / 2)) ^ 2
  RL2 = ATAN2(SR, CR)
  TNHI = SIN(CAPI / 2) / COS(CAPI / 2)
  RAIL2 = SQR(1 - 12 * TNHI ^ 2 * COS(2 * PL2) + 36 * TNHI ^ 4)
 
  'COMPUTE M1 FACTORS - SEE MANUAL, PAGE 41 -----------------------------
  FCM1 = COSI / COS(CAPI / 2) ^ 2
  QAIM1 = SQR(.25# + 1.5# * FCM1 * COS(2 * PL2) + 2.25# * FCM1 ^ 2)
  SQ = SIN(2 * PL2)
  CQ = COS(2 * PL2) + 3 * FCM1
  QU = ATAN2(SQ, CQ)
END SUB 'UPDATE ============================================================

