DECLARE FUNCTION ACCEL# (J%, K%, R#, P#)
DECLARE FUNCTION ACCEL# (J%, K%, R#, P#)
DECLARE FUNCTION ANGRDD# (X#)           'REDUCE ANGLES TO 0 TO 360
DECLARE FUNCTION ATAN2# (S#, C#)        '2 ARGUMENT ARC TANGENT
DECLARE FUNCTION DELACC# (X#, J%, K%, R#, P#)  'ERROR IN ACCEL DUE TO M4, M6
DECLARE FUNCTION DDELDX# (X#, J%, K%, R#, P#)  'DERIV OF ERROR WRT PHASE
DECLARE SUB DIFCON ()                   'DIFFERENCE CONSTANTS INPUT
DECLARE SUB EQUARG ()                   'UPDATE EQUILIB PHASE, NODE FACTS
DECLARE SUB FILEIN (NFIN%)              'GETS STATION FILE
DECLARE SUB FILEOUT ()                  'WRITES STATION FILE TO DISK OR FILE
DECLARE SUB HRMCON ()                   'HARMONIC CONSTANT INPUT
DECLARE SUB INFERCON ()                 'ADDS INFERRED CONSTANTS
DECLARE SUB JDN2REC (DUMDATE AS ANY)    'CONFORM DATE RECORD TO JUL DAY #
DECLARE SUB MAKHDR ()                   'MAKE HEADER
DECLARE SUB MODSTRNG (X$)               'MODIFIES STRING
DECLARE SUB PRTHDR (ILN%, IPH%)         'PRINT HEADER
DECLARE SUB TCMAIN ()                   'MAIN ROUTINE - TIDE CONSTANT INPUT
DECLARE SUB UPDATE ()                   'UPDATE ASTRONOMICAL ARGUMENTS

'TIDCONX4 ----------- STORES TIDAL CONSTANTS -------------------------------
'
'STARTED        1986 JUL 09
'ADDED TO TIDES 1991 MAR 26
'TIDCON4A VERSION STARTED 1999 NOV 06
'LAST MOD       1999 NOV 06
'
'STORES HARMONIC CONSTANTS USED IN "TIDES" PROGRAM
'
'FIRST  RECORD IS DIRECTION OF FLOOD AND EBB
'SECOND RECORD IS Z0 AND KTIDTYP
'THIRD THROUGH FORTIETH ARE AMPLITUDE AND [MINUS] MODIFIED EPOCH FOR EACH
'CONSTITUENT
'FORTY-FIRST  IS TITLE
'FORTY-SECOND IS SYSTEM OF UNITS I.E. ENGLISH OR METRIC
'
'
'HARMONIC CONSTANTS FOR SUBORDINATE STATIONS COMPUTED BY APPROXIMATE
'CORRECTIONS TO CONSTANTS FOR REFERENCE STATION
'
'PHASE OF ALL CONSTITUENTS SHIFTED BY MEAN TIME SHIFT OF HIGH AND LOW
'OR MAXIMUM EBB AND FLOOD
'
'FOR ADDITIVE CONSTANTS Z0, M2 AND M4 ALTERED TO MATCH HIGHS AND LOWS
'OR MAXIMUM EBBS AND FLOODS
'
'FOR MULTIPLICATIVE CONSTANTS Z0 AND RANGE ALTERED TO MATCH NEW MEAN AND
'SPRING OR DIURNAL RANGE
'
'SPEEDS EVALUATED FOR 2000JAN00.5 = JD2451544
'
'KTIDTYP = 0 FOR TIDAL HEIGHTS
'        = 1 FOR REVERSING CURRENT STATIONS
'        = 2 FOR HYDRAULIC CURRENT STATIONS
'        = 3 FOR ROTARY    CURRENT STATIONS
'
'INFERRED CONSTITUENTS COMPUTED FROM K1, O1, M2, S2 & N2 USING FORMULAE
'FROM SP98, PAGE 79

'DIMENSIONS, AND CONSTANTS ---------------------------------------------

  DEFINT I-M: DEFDBL A-H, O-Z
 
  TYPE DATEREC
    YEAR AS INTEGER
    MO AS INTEGER
    DA AS INTEGER
    LPGR AS INTEGER
    DATESTR AS STRING * 11
    DNOJ AS DOUBLE
    DOW AS STRING * 2
    CLNDR AS STRING * 3
    TFLAG AS INTEGER
  END TYPE

  CONST PI = 3.141593, RDPD = PI / 180#, DGPR = 180# / PI

  CONST NCON = 68           'NUMBER OF CONSTITUENTS (NOT INCLUDING Z0)
  CONST KINFR = 16          'NUMBER OF CONSTITUENTS POSSIBLY INFERRED
 
  DIM SHARED STRDATEO AS DATEREC, ENDDATEO AS DATEREC

  COMMON SHARED TT, TDATE$, STRDATE0 AS DATEREC, STRDATE AS DATEREC, ENDDATE AS DATEREC, STTDST, ITRULE
  COMMON SHARED HEADER$(), DFNAM$

  COMMON SHARED FLDAZ, EBBAZ, TZ, Z0, KTIDTYP, AMPLI(), EPCHM(), INDXHC(), TITLE$, UNITS$
  COMMON SHARED NAM$(), SPEED(), PARAMS$, XUNITS$, EXTREMA$, COMMENTS$
  COMMON SHARED JCON, SMAMP, HT(), IINFR(), NOTES$

'CONSTITUENT ARRAYS ---------------------------------------------------------
  DIM SHARED NAM$(NCON), FCTND(NCON), SPEED(NCON), V0PLU(NCON)
  DIM SHARED AMPLI(NCON), EPCHM(NCON), INDXHC(NCON), ALFA(NCON), AMFC(NCON)
  DIM SHARED IINFR(KINFR)

'END TIDCONX4.MOD ==========================================================

FUNCTION ACCEL (J, K, R, P)
  'SOLVE FOR ACCELERATION BY ITERATION ------------------------------------
  DX = 10
  X = -90
  YOLD = DELACC(X, J, K, R, P)
  X = X + DX
  y = DELACC(X, J, K, R, P)
  WHILE (SGN(y) = SGN(YOLD))                        'FIND REVERSAL OF SIGN
    YOLD = y
    X = X + DX
    y = DELACC(X, J, K, R, P)
  WEND
  IF (y = 0) THEN DX = 0
  WHILE (ABS(DX) > .001)
    DXOLD = DX
    DY = DDELDX(X, J, K, R, P)
    IF (DY = 0) THEN DX = .6 * SGN(YOLD / y) * ABS(DXOLD) ELSE DX = -y / DY
    IF (ABS(DX) > .6 * ABS(DXOLD)) THEN DX = .6 * SGN(DX) * ABS(DXOLD)
    YOLD = y
    X = X + DX
    y = DELACC(X, J, K, R, P)
  WEND
  ACCEL = X
END FUNCTION 'ACCEL ======================================================

FUNCTION DDELDX (X, J, K, R, P)
  'DERIVATIVE OF ERROR IN ACCELERATION TERM - EQ. 22, 23, 30 OR 31
  DDELDX = RDPD * (COS(RDPD * X) + K * J * K * R * COS(RDPD * (P - K * X)))
END FUNCTION 'DDELX =======================================================

FUNCTION DELACC (X, J, K, R, P)
  'ERROR IN ACCELERATION - EQ. 22, 23, 30 OR 31
  DELACC = SIN(RDPD * X) - J * K * R * SIN(RDPD * (P - K * X))
END FUNCTION 'DELACC ======================================================

SUB DIFCON
  'CONSTANTS FOR SUBORDINATE STATION ---------------------------------------
  'MODIFIED EPOCHS [EPCHM()] POSITIVE DEGREES
  'SPEEDS IN RADIANS PER HOUR

  'DIFFERENCE OF TIME DIFFERENCES ACCOUNTED FOR BY ADDING M4 TO M2
  '(SHOULD MODIFY M6, M8, S4 AND S6 ALSO ????)

  STRDATE.DNOJ = 2451544#                   'START DATE = 2000 JAN 00.5
  CALL JDN2REC(STRDATE)
  ENDDATE.DNOJ = 2451544#                   'END   DATE = 2000 JAN 00.5
  CALL JDN2REC(ENDDATE)
  CALL UPDATE                               'UPDATE ELEMENTS & DERIVATIVES
  CALL EQUARG                               'COMPUTE EQUILIBRIUM ARGS & SPEEDS
 
  CLS
  LOCATE 12, 21
  PRINT "SELECT REFERENCE STATION."
  LOCATE 25, 20: PRINT "PRESS ANY KEY TO CONTINUE"
  WHILE (INKEY$ = ""): WEND
 
  TITLX$ = TITLE$                           'STORE TITLE,
  DFNAX$ = DFNAM$                           'FILENAME,
  NOTEX$ = NOTES$                           'NOTES
  COMMX$ = COMMENTS$                        'AND COMMENTS
  CALL FILEIN(1)                            'READ IN CONSTANTS FOR REF STA
  TITLE$ = TITLX$                           'RESTORE TITLE,
  DFNAM$ = DFNAX$                           'FILENAME,
  NOTES$ = NOTEX$                           'NOTES
  COMMENTS$ = COMMX$                        'AND COMMENTS
  HEADER$(2) = TITLE$                       'RESTORE HEADER

  IF (KTIDTYP = 0) THEN
    DFNAM$ = DFNAM$ + ".TD4"
    HEADER$(4) = ""
  ELSE
    DFNAM$ = DFNAM$ + ".CR4"
    HEADER$(4) = "DIRECTION AT FLOOD(+) =" + STR$(FLDAZ) + ", AT EBB(-) =" + STR$(EBBAZ) + ""
  END IF

  PRINT : PRINT "DATA STORED IN FILE ", DFNAM$
  MID$(TITLE$, 65, 12) = DFNAM$
  CALL MAKHDR                               'UPDATE HEADER
 
  AND$ = "N"
  DO WHILE (AND$ = "N")
    CLS
    CALL PRTHDR(2, 0)
    PRINT : INPUT "TIME DIFFERENCE FOR HIGH/FLOOD (H:MM) = ? ", DTHI$
    PRINT : INPUT "TIME DIFFERENCE FOR LOW/EBB    (H:MM) = ? ", DTLO$
    PRINT

    DTHI = VAL(LEFT$(DTHI$, 2))
    DTHM = VAL(MID$(DTHI$, 4, LEN(DTHI$) - 3)) / 60
    IF (LEFT$(DTHI$, 1) = "-") THEN DTHI = DTHI - DTHM ELSE DTHI = DTHI + DTHM
    DTLO = VAL(LEFT$(DTLO$, 2))
    DTLM = VAL(MID$(DTLO$, 4, LEN(DTLO$) - 3)) / 60
    IF (LEFT$(DTLO$, 1) = "-") THEN DTLO = DTLO - DTLM ELSE DTLO = DTLO + DTLM
    DTAV = (DTHI + DTLO) / 2                  'AVERAGE TIME SHIFT
    DDT2 = (DTHI - DTLO) / 2                  'DEVIATION  FROM AVERAGE

    IF (KTIDTYP = 0) THEN
      INPUT "ARE HEIGHT DIFFERENCES GIVEN AS A RATIO, I.E. MARKED BY * ? (Y/N) ", ANS$
      ANSTAR$ = UCASE$(LEFT$(ANS$, 1))
      PRINT
    END IF
  
    SELECT CASE KTIDTYP
      CASE 0
        IF (ANSTAR$ = "Y") THEN
         'COMPUTATION FOR MULTIPLICATIVE HEIGHT CORRECTIONS -----------------------
          PRINT "ENTER RANGES AND MEAN TIDE LEVEL IN " + UNITS$ + " UNITS.": PRINT
          INPUT "SPRING OR DIURNAL RANGE AT SUBORDINATE STATION = ? ", RNGS: PRINT
          INPUT "MEAN TIDE LEVEL         AT SUBORDINATE STATION = ? ", Z0: PRINT
          INPUT "SPRING OR DIURNAL RANGE AT REFERENCE   STATION = ? ", RNGR: PRINT
        ELSE
          'COMPUTATION FOR ADDITIVE HEIGHT CORRECTIONS -----------------------------
          PRINT "ENTER HEIGHT DIFFERENCES IN " + UNITS$ + " UNITS.": PRINT
          INPUT "HEIGHT DIFFERENCE FOR HIGH = ? ", DHHI
          INPUT "HEIGHT DIFFERENCE FOR LOW  = ? ", DHLO: PRINT
        END IF
      CASE ELSE
        'COMPUTATION FOR SPEED RATIOS --------------------------------------------
        PRINT "DATA AT SUBORDINATE STATION -": PRINT
        INPUT "SPEED   OF AVERAGE MAXIMUM FLOOD (KNOTS) = ? ", FMXS
        INPUT "DIRECTION OF AVERAGE MAXIMUM FLOOD (DEG)   = ? ", FLDAZ
        INPUT "SPEED   OF AVERAGE MAXIMUM EBB   (KNOTS) = ? ", EMXS
        INPUT "DIRECTION OF AVERAGE MAXIMUM EBB   (DEG)   = ? ", EBBAZ: PRINT
        PRINT "DATA AT REFERENCE  STATION -"
        INPUT "SPEED   OF AVERAGE MAXIMUM FLOOD (KNOTS) = ? ", FMXR
        INPUT "SPEED   OF AVERAGE MAXIMUM EBB   (KNOTS) = ? ", EMXR: PRINT
    END SELECT
 
    INPUT "WERE ALL INPUTS CORRECT (Y/N) "; AND$
    AND$ = UCASE$(LEFT$(AND$, 1))
  LOOP
  
  FOR J = 1 TO JCON                         'CORRECT PHASES FOR MEAN DELAY
    I = INDXHC(J)
    IF (AMPLI(I) <> 0) THEN
      EPCHM(I) = ANGRDD(EPCHM(I) + DTAV * SPEED(I) * DGPR)
    ELSE
      EPCHM(I) = 0
    END IF
  NEXT J
 
  SELECT CASE KTIDTYP
    CASE 0
      IF (ANSTAR$ = "Y") THEN
        'COMPUTATION FOR MULTIPLICATIVE HEIGHT CORRECTIONS -----------------------
        FACTA = RNGS / RNGR
        FOR J = 1 TO JCON
          I = INDXHC(J)
          AMPLI(I) = FACTA * AMPLI(I)
        NEXT J
      ELSE
        'COMPUTATION FOR ADDITIVE HEIGHT CORRECTIONS -----------------------------
        Z0 = Z0 + (DHHI + DHLO) / 2
        DDH2 = (DHHI - DHLO) / 2
        AMPLI(1) = AMPLI(1) + DDH2  'M2 ONLY MODIFIED
      END IF
    CASE ELSE
      'COMPUTATION FOR SPEED RATIOS --------------------------------------------
      Z0 = (FMXS - EMXS) / 2
      FACTA = (FMXS + EMXS) / (FMXR + EMXR)
      IF (FACTA < 0) THEN      'HANDLES NEGATIVE EBB/FLOOD = CONTINUOUS
        ADDPHS = 180           'FLOOD/EBB
        FACTA = ABS(FACTA)
      ELSE
        ADDPHS = 0
      END IF
      
      IF (KTIDTYP = 2) THEN FACTA = FACTA * FACTA  'HYDRAULIC CURRENT STATION
     
      FOR J = 1 TO JCON
        I = INDXHC(J)
        AMPLI(I) = FACTA * AMPLI(I)
      NEXT J

  END SELECT
    
  'RECOMPUTE M2 AND M4 TO CORRECT TIMES ----------------------------------
  IF (DDT2 <> 0) THEN
   
    JM4 = 0
    FOR J = 1 TO JCON
      IF (INDXHC(J) = 5) THEN JM4 = J: EXIT FOR
    NEXT J
   
    IF (JM4 = 0) THEN                         'NO M4 IN STATION SET - ADD
      JCON = JCON + 1
      INDXHC(JCON) = 5
      AMPLI(5) = 0
      EPCHM(5) = 0
    END IF
   
    DTHT1 = DDT2 * SPEED(1)                 'DEVIATION IN PHASE - M2
    DTHT2 = DDT2 * SPEED(5)                 'DEVIATION IN PHASE - M4
    DA4 = SIN(DTHT1) / (2 * COS(DTHT2))       '(ADDED M4) / M2
    AM2P = COS(DTHT1) + DA4 * SIN(DTHT2)      'NEW MAX/ M2 AMPL
    AMPLI(1) = AMPLI(1) / AM2P         'NEW AMPLITUDE OF M2 AFTER ADD M4
    PA4 = (2 * EPCHM(1) + 90) * RDPD     'PHASE OF ADDITION TO M4
    SM4 = AMPLI(5) * SIN(RDPD * EPCHM(5)) + DA4 * AMPLI(1) * SIN(PA4)
    CM4 = AMPLI(5) * COS(RDPD * EPCHM(5)) + DA4 * AMPLI(1) * COS(PA4)
    AMPLI(5) = SQR(SM4 * SM4 + CM4 * CM4)
    EPCHM(5) = ANGRDD(ATAN2(SM4, CM4) / RDPD)
  END IF

  IF ((AMPLI(1) < 0) OR (AMPLI(1) > 99) OR (AMPLI(5) < 0) OR (AMPLI(5) > 99)) THEN
    PRINT
    PRINT "RESULTS NOT VALID -- DIFFERENCE OF TIME DIFFERENCES TOO LARGE??"
    PRINT "              PROGRAM WILL BE ABORTED"
    PRINT
    WHILE (INKEY$ = ""): WEND
    END
  END IF
     
  'CORRECT MEAN TIDE LEVEL TO MEAN WATER LEVEL ---------------------------
  'BASED ON "MANUAL OF HARMONIC CONSTANT REDUCTIONS",
  'COAST AND GEODETIC SURVEY SPECIAL PUBLICATION NO. 260, 1952
  'ABSTRACTED FROM PROGRAM HRMCONRD.BAS
  IF (ANSTAR$ = "Y") THEN             'CORRECT MEAN TIDE TO MEAN WATER lEVEL
    P4 = ANGRDD(2 * EPCHM(1) - EPCHM(5))                '2*M2-M4
    P6 = ANGRDD(3 * EPCHM(1) - EPCHM(7))                '3*M2-M6
    R4 = AMPLI(5) / AMPLI(1)                            'M4/M2
    R6 = AMPLI(7) / AMPLI(1)                            'M6/M2
    RS2M2 = AMPLI(2) / AMPLI(1)                         'S2/M2
    SK1O1 = AMPLI(4) + AMPLI(6)                         'K1+O1
    RKOM2 = SK1O1 / AMPLI(1)                            '(K1+O1)/M2
    IF ((AMPLI(6) = 0) OR (AMPLI(4) = 0)) THEN
      RO1K1 = .24                                       'NO DIURNAL EFFECT
    ELSE
      RO1K1 = AMPLI(6) / AMPLI(4)                       'O1/K1
      IF (RO1K1 > 1) THEN RO1K1 = 1 / RO1K1             'OR K1/O1
    END IF
    RN2M2 = AMPLI(3) / AMPLI(1)                         'N2/M2
  'HIGH AND LOW WATER ACCELERATIONS --------------------------------------
    J = 1: K = 2: R = R4: P = P4                     'EQUATION (22)
    V1 = ACCEL(J, K, R, P)
    J = -1: K = 2: R = R4: P = P4                   'EQUATION (23)
    W1 = ACCEL(J, K, R, P)
    J = 1: K = 3: R = R6: P = ANGRDD(P6 - 3 * V1)   'EQUATION (30)
    V2 = ACCEL(J, K, R, P)
    V = ANGRDD(V1 + V2)
    J = 1: K = 3: R = R6: P = ANGRDD(P6 - 3 * W1)   'EQUATION (31)
    W2 = ACCEL(J, K, R, P)
    W = ANGRDD(W1 + W2)
  'MEAN HIGH AND LOW WRT. MEAN SEA LEVEL ------------------- ------------
    XMH = AMPLI(1) * COS(V * RDPD)       'EQUATION (43) - HIGH HALF
    XMH = XMH + AMPLI(1) * (.01 + .2885 * RS2M2 ^ 2)
    XMH = XMH + AMPLI(1) * .036 * RKOM2 ^ 2
    XMH = XMH + AMPLI(5) * COS((P4 - 2 * V) * RDPD)
    XMH = XMH + AMPLI(7) * COS((P6 - 3 * V) * RDPD)
    XML = -AMPLI(1) * COS(W * RDPD)      'EQUATION (43) - LOW  HALF
    XML = XML - AMPLI(1) * (.01 + .2885 * RS2M2 ^ 2)
    XML = XML - AMPLI(1) * .036 * RKOM2 ^ 2
    XML = XML + AMPLI(5) * COS((P4 - 2 * W) * RDPD)
    XML = XML - AMPLI(7) * COS((P6 - 3 * W) * RDPD)
  'DIURNAL INEQUALITIES ------------------------------- EQUATION (87) -------
    X2MKO = EPCHM(1) - EPCHM(4) - EPCHM(6)
    WHILE (ABS(X2MKO) > 720)
      X2MKO = X2MKO - 720 * SGN(X2MKO)
    WEND
    DWL = (XMH + XML) / 2 - .031 * (1 - EXP(-RO1K1 / .24)) * RKOM2 * SK1O1 * COS(X2MKO * RDPD)
    Z0 = Z0 - DWL
  END IF

  IF (ADDPHS > 0) THEN                 'REVERSE PHASE IF FACTA WAS NEGATIVE
    FOR J = 1 TO JCON
      I = INDXHC(J)
      IF (AMPLI(I) > 0) THEN EPCHM(I) = ANGRDD(EPCHM(I) + ADDPHS)
    NEXT J
  END IF
  PRINT

END SUB 'DIFCON ============================================================

DEFINT N
SUB FILEOUT
  'WRITES STATION FILE TO DISK OR FILE
  'ANGRDD ADDED 1994 FEB 03 - TIDES4 FORMAT 1996 OCT 25

  OPEN DFNAM$ FOR OUTPUT AS #1
  PRINT #1, CHR$(34) + TITLE$ + CHR$(34)
  PRINT #1, CHR$(34) + NOTES$ + CHR$(34)
  PRINT #1, CHR$(34) + COMMENTS$ + CHR$(34)
  PRINT #1, USING "\     \ "; UNITS$;
  PRINT #1, USING "## "; KTIDTYP;
  PRINT #1, USING "### "; FLDAZ; EBBAZ;
  PRINT #1, USING "+##.##"; TZ
  PRINT #1, USING "Z0       ##.####"; Z0
  FOR J = 1 TO JCON
    I = INDXHC(J)
    PRINT #1, USING "\      \ "; NAM$(I);
    PRINT #1, USING "##.#### "; AMPLI(I);
    PRINT #1, USING "###.## "; EPCHM(I)
  NEXT J
  CLOSE #1

  OPEN "STALIST.T&C" FOR APPEND AS #2
  PRINT #2, CHR$(34); TITLE$; CHR$(34)
  CLOSE #2

END SUB 'FILEOUT ===========================================================

DEFSNG N
SUB HRMCON
  'HARMONIC CONSTANT DATA --------------------------------------------------
  CLS
 
  LOCATE 25, 10
  PRINT "ENTER 'QQQ' FOR CONSTITUENT NAME TO EXIT INPUT LOOP";
 
  FOR I = 1 TO NCON                                       'CLEAR ARRAYS
    AMPLI(I) = 0
    EPCHM(I) = 0
  NEXT I

  LOCATE 1, 20: PRINT "TIDAL CONSTANT PROGRAM", TDATE$
  LOCATE 3, 1: PRINT TITLE$
  LOCATE 5, 20: PRINT "     HARMONIC CONSTANT DATA"

  'PRINT USING "TZ = +##.##"; TZ
 
  PRINT : INPUT "CONSTANTS FOR TIDES OR CURRENTS (T/C) ? ", ANS$
  ANS$ = UCASE$(LEFT$(ANS$, 1))
  IF (ANS$ = "T") THEN
    KTIDTYP = 0
  ELSE
    INPUT "IS CURRENT NORMAL, HYDRAULIC, OR ROTARY (N/H/R) ? ", ANS$
    ANS$ = UCASE$(LEFT$(ANS$, 1))
    SELECT CASE ANS$
      CASE IS = "H"
        KTIDTYP = 2
      CASE IS = "R"
        KTIDTYP = 3
      CASE ELSE
        KTIDTYP = 1
    END SELECT
  END IF
 
  IF (KTIDTYP = 0) THEN DFNAM$ = DFNAM$ + ".TD4" ELSE DFNAM$ = DFNAM$ + ".CR4"
  PRINT : PRINT "DATA STORED IN FILE ", DFNAM$
  MID$(TITLE$, 65, 12) = DFNAM$
 
  IF (KTIDTYP = 0) THEN PRINT : INPUT "IS DATA IN ENGLISH OR METRIC UNITS (E/M)?  ", UNITS$
  A$ = UCASE$(LEFT$(UNITS$, 1))
  IF (A$ = "E") THEN UNITS$ = "ENGLISH" ELSE UNITS$ = "METRIC"
 
  FLDAZ = 0: EBBAZ = 0
  SELECT CASE KTIDTYP
    CASE 1, 2                                              'NORMAL , HYDRAULIC
      PRINT : INPUT "DIRECTION OF CURRENT AT FLOOD = ? ", FLDAZ   '
      PRINT : INPUT "DIRECTION OF CURRENT AT EBB   = ? ", EBBAZ
    CASE 3                                                 'ROTARY CURRENT
      PRINT : INPUT "DIRECTION OF FIRST  COMPONENT = ? ", FLDAZ
      PRINT : INPUT "DIRECTION OF SECOND COMPONENT = ? ", EBBAZ
  END SELECT
  PRINT
 
  SELECT CASE KTIDTYP
    CASE 0
      PRINT "Z0 (= H0)  (IN REMARKS COLUMN OF NOS FORM 444)"
    CASE IS > 0
      PRINT "Z0 = PERMANENT CURRENT (IN REMARKS COLUMN OF NOS FORM 444)"
  END SELECT

  PRINT
  PRINT "NAME     AMPLI  EPCHM"
  PRINT "         B (H)  k' (g)"
  INPUT "Z0       ", Z0
  JCON = 0
  DO
    IFLAG = 0                                     'NAME NOT FOUND IN HC LIST
    INPUT ; "", NAMT$
    NAMT$ = UCASE$(NAMT$)
    IF (LEFT$(NAMT$, 3) = "QQQ") THEN EXIT DO
    LOCATE CSRLIN, 10
    INPUT ; "", AMPLIT
    LOCATE CSRLIN, 18
    INPUT "", EPCHMT
    FOR I = 1 TO NCON                             'SEARCH CONSTITUENT LIST
      IF (NAM$(I) = NAMT$) THEN                   'NAME FOUND IN HC LIST
        JCON = JCON + 1
        INDXHC(JCON) = I                          'CONST INDEX FOR JTH INPUT
        AMPLI(I) = AMPLIT
        EPCHM(I) = EPCHMT
        IFLAG = 1
        EXIT FOR
      END IF
    NEXT I

    IF (IFLAG = 0) THEN
      PRINT "CONSTITUENT NAME '"; NAMT$; "' NOT FOUND IN PRESENT SET"
    END IF

  LOOP

  PRINT
  PRINT "NUMBER OF CONSTITUENTS IN CURRENT SET = "; JCON
  PRINT

  'REVIEW AND CHECK DATA
  ANS$ = "N"
  DO WHILE (ANS$ <> "Y")
    CLS
    PRINT TITLE$
    PRINT NOTES$
    PRINT COMMENTS$
    PRINT
    PRINT USING "Z0       ##.####    "; Z0
    FOR J = 1 TO JCON STEP 2
      I = INDXHC(J)
      PRINT USING "\      \ "; NAM$(I);
      PRINT USING "##.#### "; AMPLI(I);
      PRINT USING "###.## "; EPCHM(I);
     
      IF (J < JCON) THEN                        'MODIFY FOR ROTARY CURRENTS
        I = INDXHC(J + 1)
        PRINT "    "
        PRINT USING "\      \ "; NAM$(I);
        PRINT USING "##.#### "; AMPLI(I);
        PRINT USING "###.## "; EPCHM(I);
      END IF

      IF (CSRLIN = 24) THEN
        LOCATE 25, 20
        PRINT "PRESS ANY KEY TO CONTINUE";
        WHILE (INKEY$ = ""): WEND
        IF (J < JCON) THEN
          CLS
          PRINT "NAME     AMPLI  EPCHM"
          PRINT
        END IF
      ELSE                                        'CSRLIN < 24
        PRINT
      END IF
      
    NEXT J

    INPUT "IS THIS CORRECT (Y/N) "; ANS$
    ANS$ = UCASE$(LEFT$(ANS$, 1))
    IF (ANS$ <> "N") THEN EXIT DO
    INPUT "Z0 OR CONSTITUENT VALUE (Z/C) "; AN2$
    AN2$ = UCASE$(LEFT$(AN2$, 1))
    SELECT CASE AN2$
    CASE "Z"
      INPUT "Z0 = "; Z0
    CASE "C"
      INPUT "CONST. NAME, AMPLITUDE, EPOCH' = "; CNAM$, AMPLN, EPCHN
      FOR J = 1 TO JCON
        I = INDXHC(J)
        IF (CNAM$ = NAM$(I)) THEN EXIT FOR
      NEXT J
      IF (J <= JCON) THEN
        AMPLI(I) = AMPLN
        EPCHM(I) = ANGRDD(EPCHN)
      ELSE
        PRINT
        PRINT "CONSTITUENT NOT FOUND - PRESS ANY KEY TO CONTINUE"
        WHILE (INKEY$ = ""): WEND
      END IF
    END SELECT
  LOOP
 
  PRINT : INPUT "ADD INFERRED CONSTITUENTS (Y/N) ? ", ANS$
  ANS$ = UCASE$(LEFT$(ANS$, 1))
  IF (ANS$ = "Y") THEN CALL INFERCON

END SUB 'HRMCON ===========================================================

SUB INFERCON
  'ADD INFERRED CONSTANTS - SEE MANUAL P. 79 -----------------------------
  'LAST MOD 1996 OCT 28
  'M2, S2, K1 & O1 ASSUMED IN ORIGINAL STATION SET

  'CHECK FOR POSSIBLE INFERABLE CONSTITUENTS IN STATION SET
  FOR K = 1 TO KINFR
    I = IINFR(K)                         'INDEX OF INFERABLE CONST
    JFLAG = 0                            'INFERABLE CONST NOT IN STATION SET
    FOR J = 1 TO JCON
      IF (INDXHC(J) = I) THEN            'INFERABLE CONST IN STATION SET
        JFLAG = 1
        EXIT FOR
      END IF
    NEXT J
    IF (JFLAG = 0) THEN                  'ADD TO STATION SET
      JCON = JCON + 1
      INDXHC(JCON) = I
    END IF
  NEXT K
       
  HO1 = AMPLI(6): GO1 = EPCHM(6): HK1 = AMPLI(4): GK1 = EPCHM(4)
  GO1 = GK1 + ANGRDD(GO1 - GK1 + 180) - 180      'GO1 WITHIN 180 OF GK1
  IF (AMPLI(19) = 0) THEN AMPLI(19) = .079 * HO1: EPCHM(19) = GK1 + .496 * (GK1 - GO1)
  IF (AMPLI(18) = 0) THEN AMPLI(18) = .071 * HO1: EPCHM(18) = GK1 - .5 * (GK1 - GO1)
  IF (AMPLI(15) = 0) THEN AMPLI(15) = .043 * HO1: EPCHM(15) = GK1 + 1# * (GK1 - GO1)
  IF (AMPLI(30) = 0) THEN AMPLI(30) = .331 * HK1: EPCHM(30) = GK1 - .075 * (GK1 - GO1)
  IF (AMPLI(26) = 0) THEN AMPLI(26) = .194 * HO1: EPCHM(26) = GK1 - 1.496 * (GK1 - GO1)
  IF (AMPLI(29) = 0) THEN AMPLI(29) = .026 * HO1: EPCHM(29) = GK1 - 1.992 * (GK1 - GO1)
  IF (AMPLI(25) = 0) THEN AMPLI(25) = .038 * HO1: EPCHM(25) = GK1 - 1.429 * (GK1 - GO1)
  HM2 = AMPLI(1): GM2 = EPCHM(1): HS2 = AMPLI(2): GS2 = EPCHM(2): HN2 = AMPLI(3): GN2 = EPCHM(3)
  GM2 = GS2 + ANGRDD(GM2 - GS2 + 180) - 180      'GM2 WITHIN 180 OF GS2
  IF (AMPLI(35) = 0) THEN AMPLI(35) = .272 * HS2: EPCHM(35) = GS2 + .081 * (GS2 - GM2)
  IF (AMPLI(28) = 0) THEN AMPLI(28) = 8.000001E-03 * HS2: EPCHM(28) = GS2 + .04 * (GS2 - GM2)
  IF (AMPLI(27) = 0) THEN AMPLI(27) = .059 * HS2: EPCHM(27) = GS2 - .04 * (GS2 - GM2)
  IF (AMPLI(16) = 0) THEN AMPLI(16) = .007 * HM2: EPCHM(16) = GS2 - .536 * (GS2 - GM2)
  IF (AMPLI(13) = 0) THEN AMPLI(13) = .024 * HM2: EPCHM(13) = GS2 - 2# * (GS2 - GM2)
  IF (AMPLI(3) = 0) THEN                                      'H(N2) = 0
    AMPLI(3) = .194 * HM2: EPCHM(3) = GS2 - 1.536 * (GS2 - GM2)
    IF (AMPLI(33) = 0) THEN AMPLI(33) = .028 * HM2: EPCHM(33) = GS2 - .464 * (GS2 - GM2)
    IF (AMPLI(14) = 0) THEN AMPLI(14) = .026 * HM2: EPCHM(14) = GS2 - 2.072 * (GS2 - GM2)
    IF (AMPLI(11) = 0) THEN AMPLI(11) = .038 * HM2: EPCHM(11) = GS2 - 1.464 * (GS2 - GM2)
  ELSE                                                        'H(N2) > 0
    GN2 = GM2 + ANGRDD(GN2 - GM2 + 180) - 180      'GN2 WITHIN 180 OF GM2
    IF (AMPLI(33) = 0) THEN AMPLI(33) = .143 * HN2: EPCHM(33) = GM2 + 1# * (GM2 - GN2)
    IF (AMPLI(14) = 0) THEN AMPLI(14) = .133 * HN2: EPCHM(14) = GM2 - 2# * (GM2 - GN2)
    IF (AMPLI(11) = 0) THEN AMPLI(11) = .194 * HN2: EPCHM(11) = GM2 - .866 * (GM2 - GN2)
  END IF
  FOR I = 1 TO NCON
    EPCHM(I) = ANGRDD(EPCHM(I))
    IF (AMPLI(I) < .0005) THEN AMPLI(I) = 0: EPCHM(I) = 0
  NEXT I

END SUB 'INFERCON =========================================================

SUB TCMAIN
 'MAIN PROGRAM TO CREATE NEW DATA FILES ====================================
   STRDATEO = STRDATE
   ENDDATEO = ENDDATE
  
   ANS$ = "N"
   DO
     TITLE$ = STRING$(61, "-")
     MID$(TITLE$, 41, 21) = " 0000. 'N 00000. 'W"
     CLS : LOCATE 1, 20: PRINT "TIDAL CONSTANT PROGRAM", TDATE$
     PRINT : PRINT "STORES TIDAL CONSTANTS FOR USE IN TIDES4XX PROGRAM"
     PRINT : PRINT "NAME AND LOCATION OF STATION  (MAX. 60 LETTERS) = ": PRINT
     CALL MODSTRNG(TITLE$)
     PRINT : PRINT
    
     INPUT "NEW DATA FILE NAME (EXTENSION WILL BE APPENDED) = ", DFNAM$
     WHILE (LEN(DFNAM$) > 8)
       PRINT "MAXIMUM OF 8 LETTERS IN DATA FILE NAME"
       INPUT "NEW DATA FILE NAME (EXTENSION WILL BE APPENDED) = ", DFNAM$
     WEND
    
     PRINT : PRINT

     INPUT "NOS (OR OTHER) NOTES -- PRINTED IN HEADER - "; NOTES$
     PRINT
     INPUT "COMMENTS (SOURCE, ETC.) - NOT   IN HEADER - "; COMMENTS$
     PRINT

     PRINT "IF NO TIME ZONE ENTERED, NOMINAL ZONE WILL BE USED."
     INPUT "TIME ZONE (HOURS & DECIMALS, WEST +) = "; TZ$
     IF (TZ$ <> "") THEN
       TZ = VAL(TZ$)
     ELSE
       STLON = VAL(MID$(TITLE$, 52, 3)) + VAL(MID$(TITLE$, 56, 4)) / 60
       TZ = FIX((STLON / 15) + .5)
       IF (MID$(TITLE$, 61, 1) = "E") THEN STLON = -STLON: TZ = -TZ
     END IF
     PRINT
         
     PRINT TITLE$
     PRINT NOTES$
     PRINT COMMENTS$
     PRINT DFNAM$
     PRINT USING "+##.##"; TZ
     PRINT
     INPUT "ARE THE ABOVE CORRECT (Y/N OR Q) "; ANS$
     ANS$ = UCASE$(LEFT$(ANS$, 1))
   LOOP WHILE (ANS$ = "N")
   IF (ANS$ = "Q") THEN EXIT SUB
  
   TITLE$ = TITLE$ + "--(            )"
   MID$(TITLE$, 65, LEN(DFNAM$)) = DFNAM$

  
   PRINT : INPUT "IS THE DATA HARMONIC CONSTANTS OR DIFFERENCES (H/D) ? ", ANS$
   ANS$ = UCASE$(LEFT$(ANS$, 1))
   IF (ANS$ = "H") THEN CALL HRMCON ELSE CALL DIFCON
  
   'STORE NEW CONSTANTS ---------------------------------------------------
   CALL FILEOUT
   CALL MAKHDR

   STRDATE = STRDATEO
   ENDDATE = ENDDATEO

END SUB 'TCMAIN ============================================================

